#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>

#define KEY_TYPE int
#define VALUE_TYPE float

//Räsitabeli  
typedef struct bucket 
{
	KEY_TYPE key;
	VALUE_TYPE value;
	struct bucket *next;
} bucket;

//Räsitabel
typedef struct 
{
	int k;	// Who meny?
	int n;	// Who meny elements
	bucket **buckets;	// k linked-lists  
} hash_table;

unsigned long hash(void *data, int length);
void hash_table_init(hash_table *table, int k);
void bucket_init(bucket **b, KEY_TYPE key, VALUE_TYPE value);
void hash_table_insert(hash_table *table, KEY_TYPE key, VALUE_TYPE value);	 
int hash_table_contains(hash_table *table, KEY_TYPE key); 
VALUE_TYPE hash_table_get(hash_table *table, KEY_TYPE key);
void hash_table_delete(hash_table *table);

int main()
{
	hash_table table;
	hash_table_init(&table, 1000);
	hash_table_insert(&table, 1, cosf(1.0/180*M_PI));
	hash_table_insert(&table, 2, cosf(2.0/180*M_PI));
	hash_table_insert(&table, 3, cosf(3.0/180*M_PI));
	hash_table_insert(&table, 4, cosf(4.0/180*M_PI));
	hash_table_insert(&table, 5, cosf(5.0/180*M_PI));
	printf("Does it contain 1.0: %d\n", hash_table_contains(&table, 1.0));
	printf("Does it contain 1.2: %d\n", hash_table_contains(&table, 1.2));
	printf("Does it contain3.0: %d\n", hash_table_contains(&table, 3.0));
	printf("cos 1.0: %f,real %f\n", hash_table_get(&table, 1), cosf(1.0/180*M_PI));
	printf("cos 3.0: %f,real %f\n", hash_table_get(&table, 3), cosf(3.0/180*M_PI));
	return 0;
}

unsigned long hash(void *data, int length)
{
    unsigned long hash = 5381;
    int i;
    for (i = 0; i < length; i++)
    {
		hash = ((hash << 5) + hash) + ((char *)data)[i]; /* hash * 33 + c */	
	}
    return hash;
}

void hash_table_init(hash_table *table, int k)
{
	table->buckets = malloc(sizeof(*table->buckets) * k);
	table->k = k;
	table->n = 0;
}

void bucket_init(bucket **b, KEY_TYPE key, VALUE_TYPE value)
{
	*b = malloc(sizeof(bucket));
	(*b)->key = key;
	(*b)->value = value;
	(*b)->next = NULL;
}

void hash_table_insert(hash_table *table, KEY_TYPE key, VALUE_TYPE value)
{
	unsigned long k = hash(&key, sizeof(key)) % table->k;
	if (table->buckets[k] == NULL)
	{
		bucket_init(table->buckets + k, key, value);
		return;
	}
	bucket *b = table->buckets[k];
	while (1)
	{
		if (b->key == key)
		{
			return;
		}
		if (b->next == NULL)
		{
			bucket_init(&b->next, key, value);
			return;
		}
		b = b->next;
	}
}

int hash_table_contains(hash_table *table, KEY_TYPE key)
{
	unsigned long k = hash(&key, sizeof(key)) % table->k;
	if (table->buckets[k] == NULL)
	{
		return 0;
	}
	bucket *b = table->buckets[k];
	while (1)
	{
		if (b->key == key)
		{
			return 1;
		}
		if (b->next == NULL)
		{
			return 0;
		}
		b = b->next;
	}
}

VALUE_TYPE hash_table_get(hash_table *table, KEY_TYPE key)
{
	unsigned long k = hash(&key, sizeof(key)) % table->k;
	if (table->buckets[k] == NULL)
	{
		return table->buckets[k]->value;
	}
	bucket *b = table->buckets[k];
	while (1)
	{
		if (b->key == key)
		{
			return b->value;
		}
		if (b->next == NULL)
		{
			return (VALUE_TYPE)0;
		}
		b = b->next;
	}
}

void hash_table_delete(hash_table *table)
{
	int i;
	bucket *b;
	bucket *b2;
	for (i = 0; i < table->k; i++)
	{
		b = table->buckets[i];
		while (1)
		{
			b2 = b;
			b = b->next;
			free(b2);
		}
	}
	free(table->buckets);
}
